import { Button, Checkbox, Modal, Table, Upload, Tag } from "antd"; // 添加 Tag 导入
import { ColumnsType } from "antd/es/table";
import { api, useDevice, useStaff } from "@nice/client";
import {
  useEffect,
  useState,
  useImperativeHandle,
  forwardRef,
  useRef,
} from "react";
import toast from "react-hot-toast";
import React from "react";
import { useMainContext } from "../../layout/MainProvider";
import {
  ExclamationCircleOutlined,
  ImportOutlined,
  ExportOutlined,
  DatabaseOutlined,
} from "@ant-design/icons";
import dayjs from "dayjs";
import { utils, writeFile, read } from "xlsx";
import ResourceFileList from "./components/ResourceFileList";

// 提取处理嵌套字段的函数
const getNestedValue = (record: any, dataIndex: string | string[]) => {
  if (Array.isArray(dataIndex)) {
    return dataIndex.reduce((obj, key) => obj?.[key], record);
  }
  return record[dataIndex];
};

// 添加props接口
interface DeviceTableProps {
  onSelectedChange?: (keys: React.Key[], data: any[]) => void;
}

// 使用forwardRef包裹组件
const DeviceTable = forwardRef(
  ({ onSelectedChange }: DeviceTableProps, ref) => {
    // 从全局上下文获取表单、模态框控制、搜索条件等状态
    const {
      form,
      setVisible,
      searchValue,
      formValue,
      editingRecord,
      setEditingRecord,
    } = useMainContext();

    // 设备API：获取创建设备的mutation方法
    const { create } = useDevice();
    // 描述信息模态框状态：控制可见性和内容
    const [descModalVisible, setDescModalVisible] = useState(false);
    const [pageSize, setPageSize] = useState(10);
    const [currentPage, setCurrentPage] = useState(1);
    const [currentDesc, setCurrentDesc] = useState<{
      id: string;
      title: string;
      desc: string;
    }>({ id: "", title: "", desc: "" });

    // 处理点击故障名称显示描述信息
    const handleShowDesc = (record) => {
      setCurrentDesc({
        id: record.id,
        title: record.showname || "未知故障",
        desc: record.notes || "无故障详情",
      });
      setDescModalVisible(true); // 显示模态框
    };

    // 设备数据查询：获取未删除的设备列表（包含部门信息，按创建时间倒序）
    const {
      data: devices, // 设备数据数组
      isLoading, // 加载状态
      refetch, // 手动刷新数据方法
    } = api.device.findMany.useQuery(
      {
        where: (searchValue as any) || { deletedAt: null }, // 搜索条件（默认未删除）
        include: { department: true }, // 关联查询部门信息
        orderBy: { createdAt: "desc" }, // 按创建时间倒序
      },
      { enabled: true } // 自动执行查询
    );

    // 系统类型术语查询（用于类型名称转换）
    const { data: systemTypeTerms, refetch: refetchSystemType } =
      api.term.findMany.useQuery({
        where: { taxonomy: { slug: "system_type" }, deletedAt: null }, // 筛选系统类型术语
        include: { children: true }, // 包含子级术语
        orderBy: { order: "asc" }, // 按排序字段升序
      });
      
    // 设备类型术语查询（用于类型名称转换）
    const { data: deviceTypeTerms, refetch: refetchDeviceType } =
      api.term.findMany.useQuery({
        where: { taxonomy: { slug: "device_type" }, deletedAt: null }, // 筛选设备类型术语
      });
      
    // 软删除设备的mutation方法（通过ID批量删除）
    const { mutate: softDeleteByIds } =
      api.device.softDeleteByIds.useMutation();

    // 设备数据变化时刷新术语数据（保持类型信息最新）
    useEffect(() => {
      refetchSystemType();
      refetchDeviceType();
    }, [devices]);

    // 添加部门数据查询
    const { data: departments } = api.department.findMany.useQuery(
      { where: { deletedAt: null } }, // 获取未删除的部门
      { enabled: true } // 自动执行查询
    );

    // 根据术语ID和类型获取术语名称（用于表格渲染）
    const getTermNameById = (termId, termType) => {
      if (!termId) return "未知";

      const terms =
        termType === "system_type" ? systemTypeTerms : deviceTypeTerms;
      const term = terms?.find((t) => t.id === termId);
      return term?.name || "未知";
    };
    const handleDelete = (record) => {
      Modal.confirm({
        title: "确认删除",
        icon: <ExclamationCircleOutlined />,
        content: `确定要删除故障 "${record.showname || "未命名故障"}" 吗？`,
        okText: "删除",
        okType: "danger",
        cancelText: "取消",
        onOk() {
          softDeleteByIds(
            {
              ids: [record.id],
            },
            {
              onSuccess: () => {
                toast.success("删除成功");
                refetch();
              },
              onError: (error) => {
                console.error("删除故障时出错:", error);
                toast.error("删除失败");
              },
            }
          );
        },
      });
    };
    // 在分页配置中添加事件处理
    const handlePageChange = (page: number, size: number) => {
      setCurrentPage(page);
      setPageSize(size);
    };

    const handlePageSizeChange = (current: number, size: number) => {
      setCurrentPage(1); // 重置到第一页
      setPageSize(size);
    };

    const columns: ColumnsType<any> = [
      {
        title: "网系类别",
        dataIndex: "systemType",
        key: "systemType",
        align: "center",
        render: (text, record) => (
          <span className="inline-block bg-blue-50 text-blue-700 px-2 py-1 rounded text-sm border border-blue-200">
            {getTermNameById(record.systemType, "system_type")}
          </span>
        ),
      },
      {
        title: "故障类型",
        dataIndex: "deviceType",
        key: "deviceType",
        align: "center",
        render: (text, record) => (
          <span className="inline-block bg-gray-50 text-gray-700 px-2 py-1 rounded text-sm border border-gray-200">
            {getTermNameById(record.deviceType, "device_type")}
          </span>
        ),
      },
      {
        title: "单位",
        dataIndex: "deptId",
        key: "deptId",
        align: "center",
        render: (text, record) => {
          const unitName =
            (record as any)?.department?.name ||
            record.responsiblePerson ||
            "未知";
          return (
            <span className="inline-block bg-gray-50 text-gray-700 px-2 py-1 rounded text-sm border border-gray-200">
              {unitName}
            </span>
          );
        },
      },
      {
        title: "故障名称",
        dataIndex: "showname",
        key: "showname",
        align: "center",
        render: (text, record) => (
          <div
            onClick={() => handleShowDesc(record)}
            className="cursor-pointer p-2 bg-gray-50 hover:bg-blue-50 rounded border border-gray-200 hover:border-blue-300 transition-colors duration-200"
          >
            <span className="font-medium text-gray-800 hover:text-blue-700">
              {text || "未命名故障"}
            </span>
          </div>
        ),
      },
      {
        title: "故障状态",
        dataIndex: "deviceStatus",
        key: "deviceStatus",
        align: "center",
        render: (status) => {
          const statusConfig = {
            normal: {
              text: "已修复",
              className: "bg-green-100 text-green-800 border-green-200",
            },
            maintenance: {
              text: "维修中",
              className: "bg-blue-100 text-blue-800 border-blue-200",
            },
            broken: {
              text: "未修复",
              className: "bg-red-100 text-red-800 border-red-200",
            },
          };

          const config = statusConfig[status] || {
            text: "未知",
            className: "bg-gray-100 text-gray-800 border-gray-200",
          };

          return (
            <span
              className={`inline-block px-3 py-1 rounded-full text-sm font-medium border ${config.className}`}
            >
              {config.text}
            </span>
          );
        },
      },
      {
        title: "时间",
        dataIndex: "createdAt",
        key: "createdAt",
        align: "center",
        render: (text, record) => (
          <span className="text-gray-600 text-sm">
            {record.createdAt
              ? dayjs(record.createdAt).format("YYYY-MM-DD HH:mm:ss")
              : "未知"}
          </span>
        ),
      },
      {
        title: "操作",
        key: "action",
        align: "center",
        render: (_, record) => (
          <div className="flex gap-2 justify-center">
            <Button
              type="primary"
              key={record.id}
              onClick={() => handleEdit(record)}
              className="bg-blue-600 hover:bg-blue-700 border-blue-600"
              size="small"
            >
              编辑
            </Button>
            <Button danger onClick={() => handleDelete(record)} size="small">
              删除
            </Button>
          </div>
        ),
      },
    ];

    useEffect(() => {
      if (editingRecord) {
        form.setFieldsValue(editingRecord);
        console.log(editingRecord);
      }
    }, [editingRecord]);

    const handleEdit = (record) => {
      setEditingRecord(record);
      form.setFieldsValue(record);
      setVisible(true);
    };

    const [selectedRowKeys, setSelectedRowKeys] = useState<React.Key[]>([]);

    const onSelectChange = (
      newSelectedRowKeys: React.Key[],
      selectedRows: any[]
    ) => {
      setSelectedRowKeys(newSelectedRowKeys);
    };
    //导出函数
    const handleExportSelected = () => {
      // 获取选中的行数据
      const selectedData = devices?.filter((item) =>
        selectedRowKeys.includes(item.id)
      );

      if (!selectedData || selectedData.length === 0) {
        toast.error("没有选中任何数据");
        return;
      }

      try {
        // 格式化导出数据
        const exportData = selectedData.map((item) => ({
          网系类别: getTermNameById(item.systemType, "system_type"),
          故障类型: getTermNameById(item.deviceType, "device_type"),
          单位: (item as any)?.department?.name || "未知",
          故障名称: item?.showname || "未命名故障",
          故障状态: (() => {
            const statusMap = {
              normal: "已修复",
              maintenance: "维修中",
              broken: "损坏",
              idle: "闲置",
            };
            return statusMap[item.deviceStatus] || "未知";
          })(),
          时间: item.createdAt
            ? dayjs(item.createdAt).format("YYYY-MM-DD")
            : "未知",
          描述: item.notes || "无描述",
        }));

        // 创建工作簿
        const wb = utils.book_new();
        const ws = utils.json_to_sheet(exportData);
        utils.book_append_sheet(wb, ws, "故障数据");

        // 导出Excel文件
        writeFile(wb, `故障数据_${dayjs().format("YYYYMMDD_HHm  mss")}.xlsx`);

        toast.success(`成功导出 ${selectedData.length} 条数据`);
      } catch (error) {
        console.error("导出数据错误:", error);
        toast.error("导出失败");
      }
    };

    // 添加用于文件上传的ref
    const uploadRef = useRef<HTMLInputElement>(null);

    // 处理文件选择
    const handleFileChange = async (e: React.ChangeEvent<HTMLInputElement>) => {
      const files = e.target.files;
      if (!files || files.length === 0) return;

      const file = files[0];
      try {
        // 读取Excel/CSV文件
        const data = await file.arrayBuffer();
        const workbook = read(data);
        const worksheet = workbook.Sheets[workbook.SheetNames[0]];
        const jsonData = utils.sheet_to_json(worksheet);

        // 转换为符合系统格式的数据
        const records = jsonData.map((row: any) => {
          const unitName = row["单位"] ? String(row["单位"]) : "";
          return {
            systemType: findTermIdByName(row["网系类别"], "system_type"),
            deviceType: findTermIdByName(row["故障类型"], "device_type"),
            showname: row["故障名称"] ? String(row["故障名称"]) : "",
            // 尝试关联部门ID
            deptId: unitName ? findDeptIdByName(unitName) : null,
            // 同时保存原始单位名称
            responsiblePerson: unitName,
            deviceStatus: getStatusKeyByValue(row["故障状态"]),
            notes: row["描述"] ? String(row["描述"]) : "",
          };
        });
        // 确认是否有有效数据
        if (records.length === 0) {
          toast.error("未找到有效数据");
          return;
        }
        // 批量创建记录
        await batchImportRecords(records);
        refetch(); // 刷新表格数据
      } catch (error) {
        console.error("导入失败:", error);
        toast.error("导入失败，请检查文件格式");
      } finally {
        // 清空文件输入，允许再次选择同一文件
        if (uploadRef.current) uploadRef.current.value = "";
      }
    };

    // 查找术语ID
    const findTermIdByName = (
      name: string,
      type: "system_type" | "device_type"
    ) => {
      if (!name) return null;
      const terms = type === "system_type" ? systemTypeTerms : deviceTypeTerms;
      const term = terms?.find((t) => t.name === name);
      return term?.id || null;
    };

    // 改进查找部门ID函数
    const findDeptIdByName = (name: string) => {
      if (!name || name === "未知") return null;
      // 直接从部门数据中查找
      const department = departments?.find((dept) => dept.name === name);
      if (department) {
        return department.id;
      }
      // 备用：尝试从设备中查找
      const matchedDevice = devices?.find(
        (device) => (device as any)?.department?.name === name
      );
      return matchedDevice?.deptId || null;
    };
    // 获取状态键
    const getStatusKeyByValue = (value: string) => {
      const statusMap: Record<string, string> = {
        已修复: "normal",
        维修中: "maintenance",
        "未修复 ": "broken",
      };
      return statusMap[value] || "normal";
    };
    // 改进批量导入记录函数
    // 改进批量导入记录函数，使用分批处理
    const batchImportRecords = async (records: any[]) => {
      if (!records || records.length === 0) return;
      try {
        // 过滤出有效记录
        const validRecords = records.filter(
          (record) => record.systemType || record.deviceType || record.showname
        );

        if (validRecords.length === 0) {
          toast.error("没有找到有效的记录数据");
          return;
        }
        // 设置批处理大小
        const batchSize = 5;
        let successCount = 0;
        let totalProcessed = 0;
        // 显示进度提示
        const loadingToast = toast.loading(`正在导入数据...`);
        // 分批处理数据
        for (let i = 0; i < validRecords.length; i += batchSize) {
          const batch = validRecords.slice(i, i + batchSize);
          // 串行处理每一批数据
          for (const record of batch) {
            try {
              await create.mutateAsync(record);
              successCount++;
            } catch (error) {
              console.error(
                `导入记录失败: ${record.showname || "未命名"}`,
                error
              );
            }
          }
          totalProcessed += batch.length;
          // 更新导入进度
          toast.loading(
            `已处理 ${totalProcessed}/${validRecords.length} 条数据...`,
            { id: loadingToast }
          );
        }
        toast.dismiss(loadingToast);
        // 显示结果
        if (successCount === validRecords.length) {
          toast.success(`成功导入 ${successCount} 条数据`);
        } else {
          toast.success(
            `成功导入 ${successCount}/${validRecords.length} 条数据，部分记录导入失败`
          );
        }
        // 刷新数据
        refetch();
      } catch (error) {
        console.error("批量导入失败:", error);
        toast.error("导入过程中发生错误");
      }
    };
    // 添加导出模板功能
    const handleExportTemplate = () => {
      try {
        // 创建一个示例记录
        const templateData = [
          {
            网系类别: systemTypeTerms?.[0]?.name || "网系类别1",
            故障类型: deviceTypeTerms?.[0]?.name || "故障类型1",
            单位: "单位名称",
            故障名称: "示例故障名称",
            故障状态: "未修复", // 可选值:已修复, 维修中, 未修复
            描述: "示例描述",
          },
        ];

        // 创建工作簿
        const wb = utils.book_new();
        const ws = utils.json_to_sheet(templateData);
        utils.book_append_sheet(wb, ws, "导入模板");

        // 导出Excel文件
        writeFile(wb, `故障导入模板.xlsx`);

        toast.success("已下载导入模板");
      } catch (error) {
        console.error("导出模板失败:", error);
        toast.error("下载模板失败");
      }
    };
    // 触发文件选择
    const handleImportClick = () => {
      if (uploadRef.current) {
        uploadRef.current.click();
      }
    };
    const rowSelection = {
      selectedRowKeys,
      onChange: onSelectChange,
      columnWidth: 60,
      columnTitle: (
        <div className="flex items-center justify-center">
          <Checkbox
            indeterminate={
              selectedRowKeys?.length > 0 &&
              selectedRowKeys?.length < (devices?.length || 0)
            }
            checked={
              (devices?.length || 0) > 0 &&
              selectedRowKeys?.length === (devices?.length || 0)
            }
            onChange={(e) => {
              const checked = e.target.checked;
              const newSelectedRowKeys = checked
                ? (devices || []).map((item) => item.id)
                : [];
              onSelectChange(newSelectedRowKeys, checked ? devices : []);
            }}
          />
          <span className="ml-1 text-xs text-gray-600 whitespace-nowrap items-center">
            全选
          </span>
        </div>
      ),
      preserveSelectedRowKeys: true,
    };

    const TableHeader = () => (
      <div className="w-full flex justify-between items-center mb-4 p-4 bg-gray-50 rounded-lg border border-gray-200">
        <div className="flex items-center gap-3">
          <div className="w-8 h-8 bg-blue-600 rounded-lg flex items-center justify-center">
            <DatabaseOutlined className="text-white text-sm" />
          </div>
          <div>
            <h3 className="font-medium text-gray-800 mb-0">数据管理</h3>
            <p className="text-sm text-gray-500">
              {devices?.length ? `共 ${devices.length} 条记录` : "暂无数据"}
            </p>
          </div>
        </div>
        <div className="flex items-center gap-2">
          <input
            type="file"
            ref={uploadRef}
            onChange={handleFileChange}
            style={{ display: "none" }}
            accept=".xlsx,.xls,.csv"
          />
          <Button
            icon={<ImportOutlined />}
            onClick={handleExportTemplate}
            className="border-gray-300 hover:border-blue-500 hover:text-blue-600"
            size="small"
          >
            下载模板
          </Button>
          <Button
            icon={<ImportOutlined />}
            onClick={handleImportClick}
            className="border-gray-300 hover:border-green-500 hover:text-green-600"
            size="small"
          >
            导入数据
          </Button>
          <Button
            onClick={handleExportSelected}
            disabled={!selectedRowKeys || selectedRowKeys.length === 0}
            icon={<ExportOutlined />}
            type={selectedRowKeys?.length > 0 ? "primary" : "default"}
            className={
              selectedRowKeys?.length > 0 ? "bg-blue-600 hover:bg-blue-700" : ""
            }
            size="small"
          >
            {selectedRowKeys?.length > 0
              ? `导出 (${selectedRowKeys.length})`
              : "导出选中"}
          </Button>
        </div>
      </div>
    );

    return (
      <div className="space-y-4">
        <TableHeader />
        <Table
          rowSelection={rowSelection}
          rowKey="id"
          columns={columns}
          dataSource={devices}
          size="middle"
          tableLayout="fixed"
          rowClassName={(record, index) =>
            `${
              index % 2 === 0
                ? "bg-white hover:bg-blue-50"
                : "bg-gray-50 hover:bg-blue-50"
            } transition-colors duration-200`
          }
          onRow={(record) => ({
            style: { cursor: "pointer" },
          })}
          onHeaderRow={() => ({
            style: {
              backgroundColor: "#f8fafc",
            },
          })}
          scroll={{ x: "max-content" }}
          pagination={{
            position: ["bottomCenter"],
            className: "mt-4",
            pageSize: pageSize,
            current: currentPage,
            showSizeChanger: true,
            pageSizeOptions: ["10", "15", "20"],
            responsive: true,
            showTotal: (total, range) => (
              <span className="text-gray-600">
                显示第 {range[0]}-{range[1]} 条，共 {total} 条数据
              </span>
            ),
            showQuickJumper: true,
            onChange: handlePageChange,
            onShowSizeChange: handlePageSizeChange,
          }}
          bordered
          className="bg-white rounded-lg shadow-sm"
        />

        {/* 简化的模态框 */}
        <Modal
          title={
            <div className="flex items-center gap-2">
              <DatabaseOutlined className="text-blue-600" />
              <span className="text-lg font-medium text-gray-800">
                {currentDesc.title}
              </span>
            </div>
          }
          open={descModalVisible}
          onCancel={() => setDescModalVisible(false)}
          footer={[
            <Button key="close" onClick={() => setDescModalVisible(false)}>
              关闭
            </Button>,
          ]}
          width={800}
        >
          <div className="py-4 space-y-4">
            <div>
              <div className="text-base font-medium mb-3 text-gray-800">
                故障详情
              </div>
              <div className="bg-gray-50 p-4 rounded-lg border border-gray-200">
                <div className="text-gray-700 whitespace-pre-wrap leading-relaxed">
                  {currentDesc.desc}
                </div>
              </div>
            </div>

            {currentDesc?.id && (
              <div>
                <div className="text-base font-medium mb-3 text-gray-800">
                  相关附件
                </div>
                <div className="bg-white p-4 rounded-lg border border-gray-200">
                  <ResourceFileList deviceId={currentDesc.id} />
                </div>
              </div>
            )}
          </div>
        </Modal>
      </div>
    );
  }
);

export default DeviceTable;
